@@ -27,13 +27,13 @@ as their content is relevant for command line use of the `refinery`.
 Furthermore, the module documentation of `refinery.units` contains a brief
 example of how to write simple units.
 """
-__version__ = '0.4.18'
+__version__ = '0.4.19'
 __distribution__ = 'binary-refinery'
 
 from typing import Dict, Type
+from importlib import resources
 
 import pickle
-import pkg_resources
 
 from refinery.units import arg, Unit
 
@@ -42,7 +42,11 @@ def _singleton(cls):
     return cls()
 
 
-UNIT_CACHE_PATH = pkg_resources.resource_filename(__name__, '__init__.pkl')
+with resources.path(__name__, '__init__.py') as this:
+    # This is an annoying hack to allow this to work when __init__.pkl does not
+    # yet exist during setup. Starting with Python 3.9, we could use the slightly
+    # less awkward: resources.files(__name__).joinpath('__init__.pkl')
+    UNIT_CACHE_PATH = this.parent / '__init__.pkl'
 
 
 @_singleton
