@@ -93,7 +93,6 @@ from typing import Callable, Dict, Iterable, Optional, ByteString, Union
 from refinery.lib.structures import MemoryFile
 from refinery.lib.tools import isbuffer, entropy, index_of_coincidence
 from refinery.lib.mime import get_cached_file_magic_info
-from refinery.lib.loader import load_pipeline
 
 
 class CustomStringRepresentation(abc.ABC):
@@ -147,6 +146,12 @@ class ByteStringWrapper(CustomStringRepresentation):
             self._string = value = codecs.decode(self._binary, self.codec, self.error)
         return value
 
+    def __len__(self):
+        return len(self.binary)
+
+    def __iter__(self):
+        yield from self.binary
+
     def __getattr__(self, key):
         return getattr(self.binary, key)
 
@@ -158,7 +163,10 @@ class ByteStringWrapper(CustomStringRepresentation):
     def __bytes__(self):
         value = self.binary
         if isinstance(value, memoryview):
-            value = value.obj
+            if len(value) == len(value.obj):
+                return value.obj
+            else:
+                value = bytes(value)
         if isinstance(value, bytearray):
             value = bytes(value)
         return value
@@ -347,11 +355,6 @@ class LazyMetaOracle(dict, metaclass=_LazyMetaMeta):
         - `sha1`, `sha256`, and `md5` are formatted as hex strings.
         - `size` is formatted as a human-readable size with unit.
         """
-        symb = symb or {}
-        if args is None:
-            args = ()
-        elif not isinstance(args, (list, tuple)):
-            args = list(args)
         return self.format(spec, codec, args, symb, False)
 
     def format_bin(
@@ -364,21 +367,20 @@ class LazyMetaOracle(dict, metaclass=_LazyMetaMeta):
         """
         Formats the input expression using a Python F-string like expression. These strings contain
         fields in the format `{expression!T:pipeline}`, where `T` is a transformation character and
-        the `pipeline` part is any refinery pipeline as it would be specified on the command line.
-        The following transformations can be applied to the `expression` before it is (optionally)
-        processed with the given `pipeline`:
+        the `pipeline` part is a sequence of `refinery.lib.argformats.multibin` handlers which are
+        parsed in reverse. For example, the expression `{v:b64:hex}` will first decode the contents
+        of `v` using `refinery.b64`, and then decode the result using `refinery.hex`.
 
-        - `h`: decoded as a hexadecimal string
-        - `a`: endcode as latin1
-        - `s`: encoded as utf8
-        - `u`: encoded as utf16
-        - `e`: reads the input as an escaped string
+        The transformation character is only required when `expression` is a literal; it specifies
+        how to convert the literal to a binary string. The following transformations can be applied:
+
+        - `a`: literal is to be encoded using latin1
+        - `u`: literal is to be encoded using utf16
+        - `s`: literal is to be encoded using the default codec
+        - `q`: literal is a URL-encoded binary string
+        - `h`: literal is a hex-encoded binary string
+        - `e`: literal is an escaped ASCII string
         """
-        symb = symb or {}
-        if args is None:
-            args = ()
-        elif not isinstance(args, (list, tuple)):
-            args = list(args)
         return self.format(spec, codec, args, symb, True)
 
     def format(
@@ -398,9 +400,15 @@ class LazyMetaOracle(dict, metaclass=_LazyMetaMeta):
         - `refinery.lib.meta.LazyMetaOracle.format_str`
         - `refinery.lib.meta.LazyMetaOracle.format_bin`
         """
-        from refinery.lib.argformats import ParserError, PythonExpression
+        from refinery.lib.argformats import multibin, ParserError, PythonExpression
         # prevents circular import
 
+        symb = symb or {}
+        if args is None:
+            args = ()
+        elif not isinstance(args, (list, tuple)):
+            args = list(args)
+
         if fixup:
             for (store, it) in (
                 (args, enumerate(args)),
@@ -423,13 +431,13 @@ class LazyMetaOracle(dict, metaclass=_LazyMetaMeta):
 
         with stream:
             for prefix, field, modifier, conversion in formatter.parse(spec):
+                output = value = None
                 if prefix:
                     if binary:
-                        prefix = prefix.encode(codec).decode('unicode-escape').encode('latin1')
+                        prefix = prefix.encode(codec)
                     stream.write(prefix)
                 if field is None:
                     continue
-                value = None
                 if not field:
                     if not args:
                         raise LookupError('no positional arguments given to formatter')
@@ -440,6 +448,9 @@ class LazyMetaOracle(dict, metaclass=_LazyMetaMeta):
                     conversion = conversion.lower()
                     if conversion == 'h':
                         value = bytes.fromhex(field)
+                    elif conversion == 'q':
+                        from urllib.parse import unquote_to_bytes
+                        value = unquote_to_bytes(field)
                     elif conversion == 's':
                         value = field.encode(codec)
                     elif conversion == 'u':
@@ -462,7 +473,8 @@ class LazyMetaOracle(dict, metaclass=_LazyMetaMeta):
                             variables.add(field)
                 if value is None:
                     try:
-                        value = PythonExpression.evaluate(field, self)
+                        expression = PythonExpression(field, *self, *symb)
+                        value = expression(self, **symb)
                     except ParserError:
                         if not self.ghost:
                             raise KeyError(field)
@@ -474,19 +486,18 @@ class LazyMetaOracle(dict, metaclass=_LazyMetaMeta):
                         putstr('}')
                         continue
                 if binary:
-                    if isbuffer(value):
-                        output = value
-                    else:
-                        if not isinstance(value, ByteStringWrapper):
-                            value = ByteStringWrapper(str(value), codec)
-                        output = value.binary
                     modifier = modifier.strip()
                     if modifier:
-                        modifier = self.format(modifier, codec, args, symb, True, False, variables)
-                        pipeline = load_pipeline(modifier.decode(codec))
-                        output | pipeline | stream.write
-                        continue
-                else:
+                        if isinstance(value, ByteStringWrapper):
+                            value = value.binary
+                        expression = self.format(modifier, codec, args, symb, True, False, variables)
+                        output = multibin(expression.decode(codec), reverse=True, seed=value)
+                    elif isbuffer(value):
+                        output = value
+                    elif not isinstance(value, int):
+                        with contextlib.suppress(TypeError):
+                            output = bytes(value)
+                if output is None:
                     converter = {
                         'a': ascii,
                         's': str,
@@ -507,6 +518,8 @@ class LazyMetaOracle(dict, metaclass=_LazyMetaMeta):
                     else:
                         output = value
                     output = output.__format__(modifier)
+                    if binary:
+                        output = output.encode(codec)
                 stream.write(output)
             return stream.getvalue()
 
